#include <File_Archiver.h>
#include <filesystem>

using namespace inaf::oasbo::Archivers;

FileArchiver::FileArchiver(std::string parent, std::string dest) {
	this->parentPath = parent;
	this->dest = std::string(parentPath).append(dest).append(".raw");
}

FileArchiver::FileArchiver(std::string dest) {
	std::string parent = std::string(std::getenv("HOME")).append("/ArchiveRAW/");
	FileArchiver(parent,dest);
}

FileArchiver::FileArchiver() {
	std::string parent = std::string(std::getenv("HOME")).append("/ArchiveRAW/");
	std::string dest = std::string(".tmp.raw");
	FileArchiver(parent,dest);
}

int FileArchiver::write(PacketLib::BasePacket &pack) {
	if (myfile == nullptr || !myfile->is_open()) {
		std::cerr << "Archiver err: File not opened" << std::endl;
		return -1;
	}
	myfile->write((char*) pack.getBinaryPointer(),
			pack.getHeaderSize() + pack.getPayloadSize() + pack.getTailSize());
	return 1;
}

int FileArchiver::write(PacketLib::BasePacket &pack, std::string dest) {
	namespace fs = std::filesystem;
	fs::path dest_path(dest);
	if (!fs::exists(dest_path.parent_path())
			|| std::filesystem::exists(dest_path)) {
		this->dest = computeFileName(dest);
	}
	std::ofstream *file = new std::ofstream(dest, std::ios::binary);
	file->write((char*) pack.getBinaryPointer(),
			pack.getHeaderSize() + pack.getPayloadSize() + pack.getTailSize());
	file->close();
	delete file;
	return 1;
}

int FileArchiver::open() {
	namespace fs = std::filesystem;
	fs::path dest_path(this->dest);
	if (!fs::exists(dest_path.parent_path())
			|| std::filesystem::exists(dest_path)) {
		this->dest = computeFileName(this->dest);
	}
	if (myfile != nullptr)
		delete myfile;
	this->myfile = new std::ofstream(this->dest, std::ios::binary);
	return myfile->good();
}

int FileArchiver::close() {
	if (myfile != nullptr && myfile->is_open())
		myfile->close();
	if (myfile != nullptr)
		delete myfile;
	myfile = nullptr;
	return 1;
}

std::string FileArchiver::getDest() {
	return dest;
}

void FileArchiver::setDest(std::string dest) {
	this->dest = std::string(this->parentPath).append(dest).append(".raw");
}

std::string FileArchiver::computeFileName(std::string fileName) {
	namespace fs = std::filesystem;
	// Check if the directory of the file path exists
	fs::path dest_path(fileName);
	if (!fs::exists(dest_path.parent_path())) {
		// Create the directory and any missing parent directories
		fs::create_directories(dest_path.parent_path());
		return dest_path.string();
	}
	// Check if the the file already exists and append an incremental suffix
	if (fs::exists(dest_path)) {
		std::string extension = dest_path.extension().string();
		dest_path.replace_extension("");
		int count = 0;
		while (fs::exists(
				dest_path.string() + "_(" + std::to_string(++count) + ")"
						+ extension))
			;
		dest_path += "_(" + std::to_string(count) + ")" + extension;
	}
	return dest_path.string();
}

void FileArchiver::updateDest() {

}

class FileArchiver::Builder {
protected:
	FileArchiver *arch;
public:
	std::string file_name_key{"file_name"};
	std::string archive_path_key{"archive_path"};

	Builder() {
		this->reset();
	}
	Builder(std::string dest) {
		this->arch = new FileArchiver(dest);
	}
	Builder(std::string parent, std::string dest) {
		this->arch = new FileArchiver(parent, dest);
	}
	~Builder() {
		delete arch;
	}

	void reset() {
		this->arch = new FileArchiver();
	}

	Builder* configFrom(Configurators::BaseConfigurator *conf) {
		std::map<std::string, std::string> params =
				conf->readConfig();
		if (params.count(file_name_key) > 0)
			arch->parentPath = params[file_name_key];
		if (params.count(archive_path_key) > 0)
			arch->setDest(params[archive_path_key]);
		return this;
	}

	Builder* setParent(std::string parent) {
			arch->parentPath = parent;
			return this;
	}

	Builder* setDest(std::string dest) {
		arch->setDest(dest);
		return this;
	}

	FileArchiver* getArchiver() {
		FileArchiver *result = this->arch;
		this->reset();
		return result;
	}
};
